@@ -1,12 +1,10 @@
 #!/bin/bash
-# Copyright 1999-2004 Gentoo Technologies, Inc.
+# Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/depscan.sh,v 1.26 2004/04/21 17:09:18 vapier Exp $
+# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/depscan.sh,v 1.30 2004/10/31 10:43:06 azarah Exp $
 
 source /etc/init.d/functions.sh
 
-ebegin "Caching service dependencies"
-
 if [ ! -d "${svcdir}" ]
 then
 	if ! mkdir -p -m 0755 "${svcdir}" 2>/dev/null
@@ -26,8 +24,27 @@ do
 	fi
 done
 
+# Only update if files have actually changed
+update=1
+if [ "$1" == "-u" ]
+then
+	update=0
+	for config in /etc/conf.d /etc/init.d /etc/rc.conf
+	do
+		if [ "${config}" -nt "${svcdir}/depcache" ]
+		then
+			update=1
+			break
+		fi
+	done
+	shift
+fi
+[ ${update} -eq 0 ] && exit 0
+
+ebegin "Caching service dependencies"
+
 # Clean out the non volitile directories ...
-rm -rf ${svcdir}/dep{cache,tree} ${svcdir}/{broken,snapshot}/*
+rm -rf "${svcdir}"/dep{cache,tree} "${svcdir}"/{broken,snapshot}/*
 
 retval=0
 SVCDIR="${svcdir}"
@@ -44,13 +61,14 @@ cd /etc/init.d
 	retval=1
 
 bash "${svcdir}/depcache" | \
-\
 /bin/gawk \
 	-f /lib/rcscripts/awk/functions.awk \
 	-f /lib/rcscripts/awk/gendepends.awk || \
 	retval=1
 
-#eend ${retval} "Failed to cache service dependencies"
+touch -m "${svcdir}"/dep{cache,tree}
+
+eend ${retval} "Failed to cache service dependencies"
 
 exit ${retval}
 
