@@ -1,9 +1,9 @@
 #!/bin/bash
 # Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/env-update.sh,v 1.5 2004/10/18 17:35:45 vapier Exp $
+# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/env-update.sh,v 1.6 2004/12/03 03:56:17 vapier Exp $
 
-source /etc/init.d/functions.sh
+source /etc/init.d/functions.sh || exit 1
 
 if [ "${EUID}" -ne 0 ]
 then
@@ -24,21 +24,11 @@ note:
 export SVCDIR="${svcdir}"
 
 # Only update if files have actually changed
-update=1
 if [ "$1" == "-u" ]
 then
-	update=0
-	for config in /etc/env.d
-	do
-		if [ "${config}" -nt "${svcdir}/envcache" ]
-		then
-			update=1
-			break
-		fi
-	done
+	is_older_than "${svcdir}/envcache" /etc/env.d && exit 0
 	shift
 fi
-[ ${update} -eq 0 ] && exit 0
 
 if [ "$#" -ne 0 ]
 then
