@@ -1,6 +1,6 @@
 # Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.79 2004/11/08 03:48:56 agriffis Exp $
+# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.81 2004/12/03 03:56:17 vapier Exp $
 
 RC_GOT_FUNCTIONS="yes"
 
@@ -139,11 +139,10 @@ setup_defaultlevels() {
 #
 #    prints the current libdir {lib,lib32,lib64}
 #
-get_libdir()
-{
-	if [ -n "${CONF_LIBDIR_OVERRIDE}" ]; then
+get_libdir() {
+	if [ -n "${CONF_LIBDIR_OVERRIDE}" ] ; then
 		CONF_LIBDIR="${CONF_LIBDIR_OVERRIDE}"
-	elif [ -x "/usr/bin/portageq" ]; then
+	elif [ -x "/usr/bin/portageq" ] ; then
 		CONF_LIBDIR="$(/usr/bin/portageq envvar CONF_LIBDIR)"
 	fi
 	echo ${CONF_LIBDIR:=lib}
@@ -637,6 +636,40 @@ is_uml_sys() {
 	return $?
 }
 
+# bool is_in_fstab(path)
+#
+#   return 0 if path is listed in /etc/fstab
+#
+#   EXAMPLE:  if is_in_fstab /proc ; then ...
+#
+is_in_fstab() {
+	[ -n "$(awk '($2 ~ /^'${*//\//\\/}'$/) { print }' /etc/fstab)" ]
+}
+
+# bool is_older_than(reference, files/dirs to check)
+#
+#   return 0 if any of the files/dirs are newer than 
+#   the reference file
+#
+#   EXAMPLE: if is_older_than a.out *.o ; then ...
+is_older_than() {
+	local x=
+	local ref="$1"
+	shift
+
+	for x in "$@"
+	do
+		[[ ${x} -nt ${ref} ]] && return 0
+
+		if [[ -d ${x} ]]
+		then
+			is_older_than "${ref}" "${x}"/* && return 0
+		fi
+	done
+
+	return 1
+}
+
 
 ##############################################################################
 #                                                                            #
